/* Bluefish HTML Editor
 * configure.c
 *
 * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <gtk/gtk.h>
#include <string.h>
#include "config.h"
#include "init.h"
#include "debug.h"

GtkWidget *font_e, *browser_e, *weblint_e, *tabfont_e, *tabpos_c;
GtkWidget *width_s, *height_s, *tab_s;

void apply_config (GtkWidget * button, gpointer entry);

/*************************
 * 
 * start of functions
 *
 *************************/

void
configure (GtkWidget * widget, gpointer data)
{
  /* This is the callback function for a configure button */

  /* This configuration dialog is simplistic and will probably need
     a rewrite. */

  /* declare the widgets */
  GtkWidget *dialog, *frame;
  GtkWidget *ok;
  GtkWidget *cancel;
  GtkWidget *vbox;
  GtkWidget *label, *notebook;
  GtkObject *adjust;
  GList *poplist = NULL;

  /* create the widgets */
  DEBUG_MSG ("configure, started\n");
  dialog = gtk_dialog_new ();
  gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
  gtk_signal_connect (GTK_OBJECT (dialog), "destroy", GTK_SIGNAL_FUNC
		      (gtk_main_quit), NULL);
  /* give the dialog a name */
  gtk_window_set_title (GTK_WINDOW (dialog), "Bluefish configuration");
  gtk_container_border_width (GTK_CONTAINER (dialog), 5);

  notebook = gtk_notebook_new ();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_BOTTOM);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), notebook, TRUE,
		      TRUE, 0);

  /* editor */
  frame = gtk_frame_new ("Editor options");
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox);

  label = gtk_label_new ("Default Font");
  font_e = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (font_e), cfg_editor_font);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 3);
  gtk_box_pack_start (GTK_BOX (vbox), font_e, FALSE, FALSE, 3);

  label = gtk_label_new ("Tab width");
  adjust = gtk_adjustment_new (1.0 * cfg_editor_tabwidth, 1, 20, 1, 1, 0);
  tab_s = gtk_spin_button_new (GTK_ADJUSTMENT (adjust), 0.1, 0);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 3);
  gtk_box_pack_start (GTK_BOX (vbox), tab_s, FALSE, FALSE, 3);

  label = gtk_label_new ("Editor");
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

  /* main GUI */
  frame = gtk_frame_new ("Main GUI options");
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox);

  label = gtk_label_new ("Notebook tab Font");
  tabfont_e = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (tabfont_e), cfg_tab_font);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 3);
  gtk_box_pack_start (GTK_BOX (vbox), tabfont_e, FALSE, FALSE, 3);
  label = gtk_label_new ("Notebook position");
  tabpos_c = gtk_combo_new ();
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (tabpos_c)->entry), FALSE);
  poplist = g_list_append (poplist, "top");
  poplist = g_list_append (poplist, "bottom");
  poplist = g_list_append (poplist, "left");
  poplist = g_list_append (poplist, "right");
  gtk_combo_set_popdown_strings (GTK_COMBO (tabpos_c), poplist);
  g_list_free (poplist);
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (tabpos_c)->entry), cfg_tab_pos);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 3);
  gtk_box_pack_start (GTK_BOX (vbox), tabpos_c, FALSE, FALSE, 3);
  label = gtk_label_new ("Initial width");
  adjust = gtk_adjustment_new (1.0 * main_window_w, 50, 2000, 5, 25, 0);
  width_s = gtk_spin_button_new (GTK_ADJUSTMENT (adjust), 0.1, 0);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 3);
  gtk_box_pack_start (GTK_BOX (vbox), width_s, FALSE, FALSE, 3);
  label = gtk_label_new ("Initial height");
  adjust = gtk_adjustment_new (1.0 * main_window_h, 50, 2000, 5, 25, 0);
  height_s = gtk_spin_button_new (GTK_ADJUSTMENT (adjust), 0.1, 0);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 3);
  gtk_box_pack_start (GTK_BOX (vbox), height_s, FALSE, FALSE, 3);

  label = gtk_label_new ("GUI");
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

  /* external programs */
  frame = gtk_frame_new ("External program options");
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox);

  label = gtk_label_new ("Browser command");
  browser_e = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (browser_e), cfg_browser_cline);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 3);
  gtk_box_pack_start (GTK_BOX (vbox), browser_e, FALSE, FALSE, 3);

  label = gtk_label_new ("Weblint command");
  weblint_e = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (weblint_e), cfg_weblint_cline);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 3);
  gtk_box_pack_start (GTK_BOX (vbox), weblint_e, FALSE, FALSE, 3);

  label = gtk_label_new ("External");
  gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

  DEBUG_MSG ("configure, height\n");
  ok = gtk_button_new_with_label (" Save & close ");
  cancel = gtk_button_new_with_label (" Cancel ");

  /* connect signals */
  gtk_signal_connect (GTK_OBJECT (ok), "clicked",
		      GTK_SIGNAL_FUNC (apply_config), dialog);
  gtk_signal_connect (GTK_OBJECT (cancel), "clicked",
		      GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

  /* pack the widgets */
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), ok,
		      TRUE, TRUE, 0);
  gtk_box_pack_end (GTK_BOX (GTK_DIALOG (dialog)->action_area), cancel,
		    TRUE, TRUE, 0);

  /* show the widgets */
  gtk_widget_show_all (dialog);

  gtk_grab_add (dialog);
  gtk_main ();
  gtk_grab_remove (dialog);

  gtk_object_destroy (GTK_OBJECT (dialog));

  gtk_main ();
}

void
string_apply (gchar ** config_var, GtkWidget * entry)
{
  gchar *tmpstring;

  tmpstring = gtk_entry_get_text (GTK_ENTRY (entry));
  DEBUG_MSG ("string_apply, tmpstring=%s\n", tmpstring);
  if (config_var != NULL)
    {
      g_free (*config_var);
    }
  *config_var = g_malloc (strlen (tmpstring) + 1);
  strcpy (*config_var, tmpstring);
}

void
integer_apply (gint * config_var, GtkWidget * spinbutton)
{

  *config_var = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spinbutton));
}


void
apply_config (GtkWidget * button, gpointer entry)
{
  string_apply (&cfg_browser_cline, browser_e);
  string_apply (&cfg_weblint_cline, weblint_e);
  string_apply (&cfg_editor_font, font_e);
  string_apply (&cfg_tab_font, tabfont_e);
  string_apply (&cfg_tab_pos, GTK_COMBO (tabpos_c)->entry);
  integer_apply (&main_window_w, width_s);
  integer_apply (&main_window_h, height_s);
  integer_apply (&cfg_editor_tabwidth, tab_s);
  gtk_main_quit ();
  save_config_files ();
}
